unit frmMain;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Mask, ExtCtrls, WinSVC, ServiceManager, ActnList,
  Grids;

type
  TMainForm = class(TForm)
    METime: TMaskEdit;
    LEtime: TLabel;
    ESize: TEdit;
    EExtensions: TEdit;
    LESize: TLabel;
    LEExtensions: TLabel;
    PButtons: TPanel;
    BSave: TButton;
    LLBDirs: TLabel;
    LLState: TLabel;
    LStatus: TLabel;
    SMMain: TServiceManager;
    BStopStart: TButton;
    ActionList1: TActionList;
    ASave: TAction;
    AStopStart: TAction;
    BPauseContinue: TButton;
    APauseContinue: TAction;
    BDirAdd: TButton;
    BDirEdit: TButton;
    BDirDelete: TButton;
    ADirAdd: TAction;
    ADirEdit: TAction;
    ADirDelete: TAction;
    SGDirs: TStringGrid;
    procedure FormShow(Sender: TObject);
    procedure ASaveExecute(Sender: TObject);
    procedure ASaveUpdate(Sender: TObject);
    procedure AStopStartUpdate(Sender: TObject);
    procedure AStopStartExecute(Sender: TObject);
    procedure APauseContinueUpdate(Sender: TObject);
    procedure APauseContinueExecute(Sender: TObject);
    procedure ADirAddExecute(Sender: TObject);
    procedure ADirEditExecute(Sender: TObject);
    procedure HaveDir(Sender: TObject);
    procedure ADirDeleteExecute(Sender: TObject);
  private
    FlastSTatus : DWord;
    procedure GetConfig;
    procedure GetDirList;
    procedure GetServiceStatus;
    procedure SaveConfig;
    procedure SetRowData(Index: Integer; DirName: String);
    { Private declarations }
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;

implementation

{$R *.dfm}

uses registry,cfgclean,frmDirEntry;


ResourceString
  SStop       = 'St&op';
  SSTart      = 'St&art';
  SPause      = '&Pause';
  SContinue   = '&Continue';
  SName       = 'Name';
  SPath       = 'Directory';
  SSize       = 'Size';
  SExtensions = 'Extensions';
  SConfirmDelete = 'Really delete directory "%s"?';

Procedure TMainForm.SaveConfig;

Var
  ReReadConfig : Boolean;

begin
  ReReadConfig:=False;
  With TRegistry.Create do
    Try
    Rootkey:=HKEY_LOCAL_MACHINE;
    If OpenKey(SkeyClean,true) then
      begin
      With METime do
        If Modified then
          begin
          WriteTime(SKeyTime,StrToTime(Text));
          Modified:=False;
          ReReadConfig:=True;
          end;
      With ESize do
        if Modified and (StrToIntDef(Text,-1)<>-1) then
          begin
          WriteInteger(SKeySize,StrToInt(Text));
          Modified:=False;
          end;
      With EExtensions do
        if Modified then
          begin
          WriteString(SKeyExtensions,Text);
          Modified:=False;
          end;
      end;
    Finally
      Free;
    end;
  If ReReadConfig and (FLastStatus=SERVICE_RUNNING) then
    begin
    SMMain.CustomControlService(SCleanService,ConfigControlCode);
    end;
end;

procedure TMainForm.FormShow(Sender: TObject);
begin
  SGDirs.Cells[0,0]:=SName;
  SGDirs.Cells[1,0]:=SPath;
  SGDirs.Cells[2,0]:=SSize;
  SGDirs.Cells[3,0]:=SExtensions;
  SMMAin.Connected:=True;
  GetConfig;
  GetDirList;
  GetServiceStatus;
end;

procedure TMainForm.GetConfig;

begin
  With TRegistry.Create do
    Try
      rootKey:=HKEY_LOCAL_MACHINE;
      If OpenKeyReadOnly(SKeyClean) then
        begin
        If ValueExists(SKeyTime) then
          METime.Text:=FormatDateTime('hh:nn',ReadTime(SKeyTime))
        else
          METime.Text:='00:00';
        If ValueExists(SKeySize) then
          ESize.Text:=IntToStr(ReadInteger(SKeySize));
        If ValueExists(SKeyExtensions) then
          EExtensions.Text:=ReadString(SKeyExtensions)
        end;
    Finally
      Free;
    end;
end;

procedure TMainForm.GetServiceStatus;

Var
  STatus : TServiceStatus;

begin
 With SMMain do
   Begin
   SMMain.GetServiceStatus(SCleanService,Status);
   FLastStatus:=Status.dwCurrentState;
   if FLAstStatus=SERVICE_RUNNING then
     begin
     LStatus.Font.Color:=clGreen;
     AStopStart.Caption:=SStop;
     end
   else
     begin
     LStatus.Font.Color:=clRed;
     if FLastStatus=SERVICE_STOPPED then
       AStopStart.Caption:=SStart;
     LStatus.Font.Color:=clRed;
     end;
   If FLastStatus=SERVICE_PAUSED Then
     APauseContinue.Caption:=SContinue
   else
     APAuseContinue.Caption:=SPause;
   LStatus.Caption:=ServiceStateToString(FLastStatus);
   end;
end;
procedure TMainForm.SetRowData(Index : Integer; DirName : String);

begin
  SGDirs.Cells[0,Index]:=DirName;
  SGDirs.Cells[1,Index]:='';
  SGDirs.Cells[2,Index]:='';
  SGDirs.Cells[3,Index]:='';
  With TRegistry.Create do
    Try
      rootKey:=HKEY_LOCAL_MACHINE;
      If OpenKeyReadOnly(SKeyClean+SKeyDirs+'\'+DirName) then
        begin
        If ValueExists(SKeyPath) then
          SGDirs.Cells[1,Index]:=ReadString(SKeyPath);
        if ValueExists(SKeySize) then
          SGDirs.Cells[2,Index]:=IntToStr(ReadInteger(SkeySize));
        if ValueExists(SKeyExtensions) then
          SGDirs.Cells[3,Index]:=ReadString(SKeyExtensions);
        end;
    Finally
      Free;
    end;
end;

procedure TMainForm.GetDirList;

Var
  L : TStrings;
  I : Integer;

begin
  L:=TStringList.Create;
  Try
    With TRegistry.Create do
      Try
        rootKey:=HKEY_LOCAL_MACHINE;
        If OpenKeyReadOnly(SKeyClean+SKeyDirs) then
          Try
            GetKeyNames(L);
          Finally
            CloseKey
          end;
      Finally
        Free;
      end;
    If L.Count=0 then
      SGDirs.RowCount:=2
    else
      SGDirs.RowCount:=L.Count+1;
    For I:=1 to L.Count do
      SetRowData(I,L[i-1]);
  Finally
    L.Free;
  end;
end;

procedure TMainForm.ASaveExecute(Sender: TObject);
begin
  SaveConfig;
end;

procedure TMainForm.ASaveUpdate(Sender: TObject);
begin
  (Sender as TAction).Enabled:=METime.Modified or
                               (ESize.Modified and
                                (StrToIntDef(ESize.Text,-1)<>-1)) or
                               EExtensions.Modified;
end;

procedure TMainForm.AStopStartUpdate(Sender: TObject);
begin
  (Sender as TAction).Enabled:=(FLastStatus=SERVICE_RUNNING) or
                               (FLastStatus=SERVICE_STOPPED)
end;

procedure TMainForm.AStopStartExecute(Sender: TObject);
begin
  If FLastStatus=SERVICE_RUNNING then
    SMMAin.StopService(SCleanService,False)
  else
    SMMAin.StartService(SCleanService,Nil);
  GetServiceStatus;
  Sleep(500); // Give time to stop
  GetServiceStatus;
end;

procedure TMainForm.APauseContinueUpdate(Sender: TObject);
begin
  (Sender as TAction).Enabled:=(FLastStatus=SERVICE_RUNNING) or
                               (FLastStatus=SERVICE_PAUSED)
end;

procedure TMainForm.APauseContinueExecute(Sender: TObject);
begin
  If FLastStatus=SERVICE_PAUSED then
    SMMAin.ContinueService(SCleanService)
  else
    SMMAin.PauseService(SCleanService);
  GetServiceStatus;
  Sleep(500); // Give time to stop
  GetServiceStatus;
end;

procedure TMainForm.ADirAddExecute(Sender: TObject);
begin
  With TDirEntryForm.Create(Self) do
    try
       If ShowModal=mrOK then
         begin
         SGDirs.RowCount:=SGDirs.RowCount+1;
         SetRowData(SGDirs.RowCount-1,EName.Text);
         end;
    finally
      Free;
    end;
end;

procedure TMainForm.ADirEditExecute(Sender: TObject);
begin
  With TDirEntryForm.Create(Self) do
    try
       With SGDirs do
         begin
         EName.Text:=Cells[0,Row];
         Ename.Enabled:=False;
         EPath.Text:=Cells[1,Row];
         ESize.Text:=Cells[2,Row];
         EExtensions.Text:=Cells[3,Row];
         end;
       If ShowModal=mrOK then
         begin
         SGDirs.RowCount:=SGDirs.RowCount+1;
         SetRowData(SGDirs.RowCount-1,EName.Text);
         end;
    finally
      Free;
    end;

end;

procedure TMainForm.HaveDir(Sender: TObject);
begin
  With SGDirs do
    (Sender as Taction).Enabled:=(Row>0) and (Cells[0,Row]<>'');
end;

procedure TMainForm.ADirDeleteExecute(Sender: TObject);

Var
  DN : String;

begin
  With SGDirs do
    DN:=Cells[0,Row];
  If MessageDLG(Format(SConfirmDelete,[DN]),mtConfirmation,[mbYes,mbNo],0)=mrYes then
    With TRegistry.Create do
      Try
        DeleteKey(SKeyClean+SKeyDirs+'\'+DN);
        GetDirList;
      finally
        Free;
      end;
end;

end.
