unit svcClean;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, SvcMgr, Dialogs,
  ExtCtrls, eventlog;

type
  TCleanService = class(TService)
    TClean: TTimer;
    ELClean: TEventLog;
    procedure ServiceExecute(Sender: TService);
    procedure ServiceStart(Sender: TService; var Started: Boolean);
    procedure TCleanTimer(Sender: TObject);
    procedure ServiceStop(Sender: TService; var Stopped: Boolean);
    procedure ServicePause(Sender: TService; var Paused: Boolean);
    procedure ServiceContinue(Sender: TService; var Continued: Boolean);
    procedure ServiceCreate(Sender: TObject);
  private
    { Private declarations }
    FCleanTime : TDateTime;
    FCleanThread : TThread;
    function InitTimer: Boolean;
    procedure PostError(Msg: String);
    function TicksTill(Atime: TDateTime): Integer;
    procedure StartCleanThread;
    procedure CleanThreadFinished(Sender : TObject);
    procedure PostInfo(Msg: String);
  Protected
    function DoCustomControl(CtrlCode: DWord): Boolean; override;
  public
    function GetServiceController: TServiceController; override;

    { Public declarations }
  end;

var
  CleanService: TCleanService;

implementation

{$R *.DFM}
{$R fclel.res}

uses Registry,DateUtils,thrClean,cfgClean;

ResourceString
  SErrCleanThreadRunning = 'The disk is already being cleaned';
  SErrFailedToInitTimer = 'The cleaning timer could not be initialized.';
  SFinishedClean = 'Finished cleaning disks. Cleared %d files in a total time %s.';
  SConfigChanged = 'Got configuration change message. Re-read config';

procedure ServiceController(CtrlCode: DWord); stdcall;
begin
  CleanService.Controller(CtrlCode);
end;

function TCleanService.GetServiceController: TServiceController;
begin
  Result := ServiceController;
end;

procedure TCleanService.ServiceExecute(Sender: TService);
begin
  While not Terminated do
    ServiceThread.ProcessRequests(True);
end;

procedure TCleanService.ServiceStart(Sender: TService;
  var Started: Boolean);
begin
  Started:=InitTimer;
  If Not Started then
    PostError(SErrFailedToInitTimer);
end;

procedure TCleanService.PostInfo(Msg : String);

begin
  try
    Try
      ELCLean.Active:=True;
      ELClean.Info(Msg);
    Finally
      ELClean.Active:=False;
    end;
  except
  end;
end;


procedure TCleanService.PostError(Msg : String);

begin
  try
    Try
      ELCLean.Active:=True;
      ELClean.Error(Msg);
    Finally
      ELClean.Active:=False;
    end;
  except
  end;
end;

procedure TCleanService.TCleanTimer(Sender: TObject);

begin
  If Assigned(FCleanThread) then
    PostError(SErrCleanThreadRunning);
  StartCleanThread;
  // Reset timer.
  If Not InitTimer then
    PostError(SErrFailedToInitTimer);
end;

Function TCleanService.InitTimer : Boolean;


begin
  Result:=False;
  TClean.Enabled:=False;
  With TRegistry.Create do
   Try
     ROOTKEY:=HKEY_LOCAL_MACHINE;
     If OpenKeyReadOnly(SKeyClean) then
       If ValueExists(SKeyTime) then
         begin
         FCleanTime:=ReadTime(SKeyTime);
         Result:=True;
         end;
   Finally
     Free;
   end;
  If Result then
    begin
    TClean.InterVal:=TicksTill(FCleanTime);
    TClean.Enabled:=True;
    end;
end;

Function TCleanService.TicksTill(Atime : TDateTime) : Integer;

begin
  If (Time>=ATime) then
    Result:=DateUtils.MilliSecondsBetween(Now,Tomorrow+Atime)
  else
    Result:=DateUtils.MilliSecondsBetween(Time,Atime);
end;

procedure TCleanService.StartCleanThread;

begin
  FCleanThread:=TCleanThread.Create(True);
  FCleanThread.OnTerminate:=Self.CleanThreadFinished;
  (FCleanThread as TCleanThread).OnErrorMessage:=PostError;
  (FCleanThread as TCleanThread).OnInfoMessage:=PostInfo;
  FCleanThread.Resume;
end;

procedure TCleanService.CleanThreadFinished(Sender : TObject);

begin
  With FCleanThread as TCleanThread do
    PostInfo(Format(SFinishedClean,[FTotalFiles,FormatDateTime('hh:nn:ss',FTotalTime)]));
  FCleanThread:=Nil;
end;


procedure TCleanService.ServiceStop(Sender: TService;
  var Stopped: Boolean);
begin
  TClean.Enabled:=False;
  If Assigned(FCleanThread) then
    With FCleanThread do
      begin
      If Suspended then
        Resume;
      Terminate;
      WaitFor;
      end;
  Stopped:=True;
end;

procedure TCleanService.ServicePause(Sender: TService;
  var Paused: Boolean);
begin
  TClean.Enabled:=False;
  If Assigned(FCleanThread) then
    FCleanThread.Suspend;
end;

procedure TCleanService.ServiceContinue(Sender: TService;
  var Continued: Boolean);
begin
  If Assigned(FCleanThread) then
    FCleanThread.Resume
  else
    InitTimer;
end;

procedure TCleanService.ServiceCreate(Sender: TObject);
begin
  ELClean.RegisterMessageFile('');
end;

function TCleanService.DoCustomControl(CtrlCode: DWord): Boolean;
begin
  Result:=(CtrlCode=ConfigControlCode);
  If Result and TClean.Enabled then
    begin
    InitTimer;
    PostInfo(SConfigChanged);
    end;
end;

end.
